"""

See options/base_options.py and options/train_options.py for more training options.

"""
import os
import time
import random
import subprocess
import numpy as np
from options.train_options import TrainOptions
from data import create_dataset
from models import create_model
from util.visualizer import Visualizer 
import torch
import torch.backends.cudnn as cudnn
def set_random_seed(seed_num):
    if seed_num != '':
        print('set random seed: ',seed_num)
        cudnn.benchmark = False      #False：不进行最优卷积搜索，以控制CUDNN种子
        cudnn.deterministic = True   #True ：调用相同的CuDNN的卷积操作，以控制CUDNN种子
        random.seed(seed_num)     #为python设置随机种子
        np.random.seed(seed_num)  #为numpy设置随机种子
        torch.manual_seed(seed_num) # 为CPU设置随机种子
        torch.cuda.manual_seed(seed_num) # 为当前GPU设置随机种子 torch.cuda.manual_seed_all(seed)   # 为所有GPU设置随机种子
if __name__ == '__main__':
    opt = TrainOptions().parse()   # get training options
    dataset = create_dataset(opt)  # create a dataset given opt.dataset_mode and other options
    dataset_size = len(dataset)    # get the number of images in the dataset.
    print('The number of training images = %d' % dataset_size)
    set_random_seed(seed_num=7654)
    model = create_model(opt)      # create a model given opt.model and other options
    model.setup(opt)               # regular setup: load and print networks; create schedulers
    visualizer = Visualizer(opt)   # create a visualizer that display/save images and plots
    total_iters = 0                # the total number of training iterations
    
    for epoch in range(opt.epoch_count, opt.n_epochs + opt.n_epochs_decay + 1):    # outer loop for different epochs; we save the model by <epoch_count>, <epoch_count>+<save_latest_freq>
        epoch_start_time = time.time()  # timer for entire epoch
        iter_data_time = time.time()    # timer for data loading per iteration
        epoch_iter = 0                  # the number of training iterations in current epoch, reset to 0 every epoch
        visualizer.reset()              # reset the visualizer: make sure it saves the results to HTML at least once every epoch
        model.update_learning_rate()    # update learning rates in the beginning of every epoch.
        for i, data in enumerate(dataset):  # inner loop within one epoch
            iter_start_time = time.time()  # timer for computation per iteration
            if total_iters % opt.print_freq == 0:
                t_data = iter_start_time - iter_data_time

            total_iters += opt.batch_size
            epoch_iter += opt.batch_size
            model.set_input(data)         # unpack data from dataset and apply preprocessing
            model.optimize_parameters()   # calculate loss functions, get gradients, update network weights

            if total_iters % opt.display_freq == 0:   # display images on visdom and save images to a HTML file
                save_result = total_iters % opt.update_html_freq == 0
                model.compute_visuals()
                visualizer.display_current_results(model.get_current_visuals(), epoch, save_result)

            if total_iters % opt.print_freq == 0:    # print training losses and save logging information to the disk
                losses = model.get_current_losses()
                t_comp = (time.time() - iter_start_time) / opt.batch_size
                visualizer.print_current_losses(epoch, epoch_iter, losses, t_comp, t_data)
                if opt.display_id > 0:
                    visualizer.plot_current_losses(epoch, float(epoch_iter) / dataset_size, losses)

            if total_iters % opt.save_latest_freq == 0:   # cache our latest model every <save_latest_freq> iterations
                print('saving the latest model (epoch %d, total_iters %d)' % (epoch, total_iters))
                save_suffix = 'iter_%d' % total_iters if opt.save_by_iter else 'latest'
                model.save_networks(save_suffix)

            iter_data_time = time.time()
        if epoch % opt.save_epoch_freq == 0:              # cache our model every <save_epoch_freq> epochs
            print('saving the model at the end of epoch %d, iters %d' % (epoch, total_iters))
            model.save_networks('latest')
            model.save_networks(epoch)
            str_epoch = str(epoch)

            # if epoch % opt.save_epoch_freq == 0:
            #     if "Brats" in opt.name:
            #         print("start run_Brats........")
            #         child = subprocess.Popen("sh /home/zjj/pertrain/torch_unet_z/run_Brats.sh %s %s" % (opt.name, str(epoch)),shell=True)
            #     else :
            #         print("start run_Hecktor........")
            #         #os.system("CUDA_VISIBLE_DEVICES=0 python /home/zjj/pertrain/torch_unet_z/train.py --img_mode='ct' --checkpoint_dir=%s --Load_pth=%s --base_lr=0.0001 --weight_decay=0.0001" % (opt.name, str(epoch)))
            #         child = subprocess.Popen("sh /home/zjj/pertrain/torch_unet_z/run.sh %s %s" % (opt.name, str(epoch)),shell=True)
            #         #os.system("sh /home/zjj/pertrain/torch_unet_z/run.sh %d %s %s" % (0,opt.name, str(epoch)))
            #         #child_pet = subprocess.Popen("CUDA_VISIBLE_DEVICES=1 python /home/zjj/pertrain/torch_unet_z/train.py --img_mode='pet' --checkpoint_dir=%s --Load_pth=%s --base_lr=0.0001 --weight_decay=0.0001" % (opt.name, str(epoch)),shell=True)
            # # else:
            # #     child = subprocess.Popen("sh /home/zjj/pertrain/torch_unet_z/run.sh %d %s %s" % (1,opt.name, str(epoch)),shell=True)
            #     #child_pet = subprocess.Popen("CUDA_VISIBLE_DEVICES=3 python /home/zjj/pertrain/torch_unet_z/train.py --img_mode='pet' --checkpoint_dir=%s --Load_pth=%s --base_lr=0.0001 --weight_decay=0.0001" % (opt.name, str(epoch)),shell=True)
        print('End of epoch %d / %d \t Time Taken: %d sec' % (epoch, opt.n_epochs + opt.n_epochs_decay, time.time() - epoch_start_time))
